from typing import Any
from cmq.aws.aws import AWSResource
from cmq.aws.resource.sns_subscription import sns_subscription


class sns(AWSResource):

    def __init__(self, parent=None):
        super().__init__(parent)
        self._service = "sns"
        self._resource = "sns"
        self._list_function = "list_topics"

        self._describe_function = "get_topic_attributes"
        self._describe_function_key = "TopicArn"
        self._describe_resource_key = "TopicArn"

        self._tag_function = "list_tags_for_resource"
        self._tag_function_key = "ResourceArn"

        self._metric_namespace = "AWS/SNS"
        self._metric_dimension_name = "TopicName"
        self._metric_dimension_resource_key = "TopicName"

    def _get_tag_resource_identifier(self, context: dict[str, Any], resource: dict[str, Any]) -> str:
        return resource["TopicArn"]

    def subscription(self, *args, **kwargs) -> AWSResource:
        return sns_subscription(self)(*args, **kwargs)



from cmq.aws.aws import AWSResource


class resource_explorer(AWSResource):
    """
    A class representing a resource explorer.

    This class extends the base Resource class and provides functionality
    for exploring resources.

    Attributes:
        _service (str): The service name for the resource explorer.
        _resource (str): The resource name for the resource explorer.
        _list_function (str): The list function name for the resource explorer.
        _list_key (str): The list key for the resource explorer.

    Methods:
        get_parameters: Get the parameters for the resource explorer.
    """

    def __init__(self, parent=None):
        super().__init__(parent)
        self._service = "resource-explorer-2"
        self._resource = "resource-explorer"
        self._list_function = "list_resources"
        self._list_key = "Resources"



import json
from cmq.aws.aws import AWSResource


class security_group(AWSResource):

    def __init__(self, parent=None):
        super().__init__(parent)
        self._service = "ec2"
        self._resource = "security-group"
        self._list_function = "describe_security_groups"



from typing import Any
from cmq.aws.aws import AWSResource
from cmq.aws.resource.rds_parameter import rds_parameter


class rds_parameter_group(AWSResource):

    def __init__(self, parent=None):
        super().__init__(parent)
        self._service = "rds"
        self._resource = "rds_parameter_group"
        self._list_function = "describe_db_parameter_groups"
        self._list_key = "DBParameterGroups"

        self._tag_function = "list_tags_for_resource"
        self._tag_function_key = "ResourceName"

    def get_parameters(self, context: dict) -> dict:
        parameters = self._list_parameters.copy()
        if 'rds_rds' in context:
            parameters["DBParameterGroupName"] = context["rds_rds"]["DBInstanceIdentifier"]
        return parameters

    def parameter(self, *args, **kwargs) -> AWSResource:
        return rds_parameter(self)(*args, **kwargs)

    def _get_tag_resource_identifier(self, context: dict[str, Any], resource: dict[str, Any]) -> str:
        return f"arn:aws:rds:{context['aws_region']}:{context['aws_account']}:pg:{resource['DBParameterGroupName']}"



from cmq.aws.aws import AWSResource

class region(AWSResource):

    def __init__(self, parent=None):
        super().__init__(parent)
        self._service = "account"
        self._resource = "region"
        self._list_function = "list_regions"
        self._list_key = "Regions"
        self._list_parameters = {"RegionOptStatusContains": ["ENABLED", "ENABLED_BY_DEFAULT"]}

    def __call__(self, regions: list[str] = [], **kwargs) -> "region":
        self._regions = regions
        self._list_parameters = kwargs
        return self

    def get(self, context: dict) -> list[dict]:
        if self._regions:
            return [{"RegionName": region} for region in self._regions]
        return super().get(context)

    def traverse(self, context: dict) -> None:
        for resource in self.bar(context, self.get(context), desc=f"{self._desc} traverse"):
            aws_region = resource.get("RegionName")
            aws_session = context.get("aws_session")
            aws_session._session.set_config_variable('region', aws_region)
            context.update({
                "aws_region": aws_region,
                "aws_session": aws_session,
                self._desc: resource
            })
            self._traverse(context)



from cmq.aws.aws import AWSResource


class user_key(AWSResource):

    def __init__(self, parent=None):
        super().__init__(parent)
        self._service = "iam"
        self._resource = "key"
        self._list_function = "list_access_keys"
        self._list_key = "AccessKeyMetadata"

    def get_parameters(self, context: dict) -> dict:
        parameters = self._list_parameters.copy()
        if 'iam_user' in context:
            parameters["UserName"] = context["iam_user"]["UserName"]
        return parameters


from cmq.aws.aws import AWSResource
from cmq.aws.resource.log_stream import log_stream
from cmq.aws.resource.log_event import log_event


class log(AWSResource):

    def __init__(self, parent=None):
        super().__init__(parent)
        self._service = "logs"
        self._resource = "log-group"
        self._list_function = "describe_log_groups"

        self._tag_function = "list_tags_for_resource"
        self._tag_function_key = "resourceArn"
        self._tag_resource_key = "logGroupArn"

    @property
    def stream(self) -> log_stream:
        return log_stream(self)

    @property
    def event(self, **kwargs) -> log_event:
        return log_event(self).__call__(**kwargs)

    def _get_tag_from_result(self, result) -> list:
        return result.get("tags", {})

    def _format_tags(self, tags) -> dict:
        return {"Tags": tags}


import json
from cmq.aws.aws import AWSResource


class resource_group(AWSResource):

    def __init__(self, parent=None):
        super().__init__(parent)
        self._service = "resource-groups"
        self._resource = "resource-group"
        self._list_function = "search_resources"

    def search(self, resource_types: list = ["AWS::AllSupported"], **tags: dict) -> "resource_group":
        self._list_parameters = {
            "ResourceQuery": {
                "Type": "TAG_FILTERS_1_0",
                "Query": json.dumps({
                    "ResourceTypeFilters": resource_types,
                    "TagFilters": [
                        {
                            "Key": key,
                            "Values": [value]
                        } for key, value in tags.items()
                    ]
                })
            }
        }
        return self


from typing import Any
from cmq.aws.aws import AWSResource


class function(AWSResource):

    def __init__(self, parent=None):
        super().__init__(parent)
        self._service = "lambda"
        self._resource = "function"
        self._list_function = "list_functions"
        self._list_key = "Functions"

        self._describe_function = "get_function"
        self._describe_function_key = "FunctionName"
        self._describe_resource_key = "FunctionName"

        self._tag_function = "list_tags"
        self._tag_function_key = "Resource"

        self._metric_namespace = "AWS/Lambda"
        self._metric_dimension_name = "FunctionName"
        self._metric_dimension_resource_key = "FunctionName"

    def _get_tag_resource_identifier(self, context: dict[str, Any], resource: dict[str, Any]) -> str:
        return resource["FunctionArn"]

    def _get_tag_from_result(self, result):
        return result.get("Tags", {})

    def _format_tags(self, tags) -> dict:
        return {"Tags": tags}


from _typeshed import Incomplete

class ResourceInterface:
    def attr(self, *args) -> Resource: ...
    def filter(self, func) -> Resource: ...
    def transform(self, key, func) -> Resource: ...
    def calculate(self, key, func) -> Resource: ...
    def eq(self, key, value) -> Resource: ...
    def ne(self, key, value) -> Resource: ...
    def in_(self, key, value) -> Resource: ...
    def contains(self, key, value) -> Resource: ...
    def not_contains(self, key, value) -> Resource: ...
    def starts_with(self, key, value) -> Resource: ...
    def ends_with(self, key, value) -> Resource: ...
    def gt(self, key, value) -> Resource: ...
    def lt(self, key, value) -> Resource: ...


class Resource:
    def list(self) -> list: ...
    def dict(self) -> dict: ...
    def csv(self, flat=False) -> str: ...
    def do(self, action: callable) -> None: ...


class Session(Resource):
    console: Incomplete



from datetime import datetime, timedelta
from cmq.aws.aws import AWSResource
from matplotlib import pyplot as plt


class metric(AWSResource):

    def __init__(self, parent=None):
        super().__init__(parent)
        self._service = "cloudwatch"
        self._resource = "metric"
        self._list_function = "get_metric_data"
        self._list_key = "MetricDataResults"

    def statistic(self, statistic: str, namespace: str, metric_name: str,
            dimensions: dict = {},
            period: int = 60*60, unit: str = "Count",
            start_time: datetime = None, end_time: datetime = None) -> 'metric':
        # Set default parameter values
        start_time = start_time or datetime.today() - timedelta(days=90)
        end_time = end_time or datetime.today()

        self._list_parameters = {
            "MetricDataQueries": [{
                "Id": f"cmq_{metric_name}",
                "MetricStat": {
                    "Metric": {
                        "Namespace": namespace,
                        "MetricName": metric_name,
                        "Dimensions": [
                            {"Name": k, "Value": v}
                            for k, v in dimensions.items()
                        ]
                    },
                    "Period": period,
                    "Stat": statistic,
                    "Unit": unit
                },
                "Label": unit,
                "ReturnData": True,
            }],
            "StartTime": start_time,
            "EndTime": end_time
        }
        return self

    def max(self, **kwargs) -> 'metric':
        return self.statistic("Maximum", **kwargs)

    def min(self, **kwargs) -> 'metric':
        return self.statistic("Minimum", **kwargs)

    def avg(self, **kwargs) -> 'metric':
        return self.statistic("Average", **kwargs)

    def sum(self, **kwargs) -> 'metric':
        return self.statistic("Sum", **kwargs)

    def plot(self, unit_factor=1) -> None:
        resources = self.dict()
        plt.clf()
        for session, metrics in resources.items():
            for metric in metrics:
                timestamps = metric["Timestamps"]
                values = [value/unit_factor for value in metric["Values"]]
                plt.plot(timestamps, values, label=session)
        plt.title(self._list_parameters["MetricDataQueries"][0]["MetricStat"]["Metric"]["MetricName"])
        plt.legend(loc='best')
        plt.show()


from typing import Any
from cmq.aws.aws import AWSResource


class rds_parameter(AWSResource):

    def __init__(self, parent=None):
        super().__init__(parent)
        self._service = "rds"
        self._resource = "rds_parameter"
        self._list_function = "describe_db_parameters"
        self._list_key = "Parameters"

    def get_parameters(self, context: dict) -> dict:
        parameters = self._list_parameters.copy()
        if 'rds_rds_parameter_group' in context:
            parameters["DBParameterGroupName"] = context["rds_rds_parameter_group"]["DBParameterGroupName"]
        return parameters



from typing import Any
from cmq.aws.aws import AWSResource


class sns_subscription(AWSResource):

    def __init__(self, parent=None):
        super().__init__(parent)
        self._service = "sns"
        self._resource = "sns_subscriptions"
        self._list_function = "list_subscriptions"

    def get_parameters(self, context: dict) -> dict:
        parameters = self._list_parameters.copy()
        if 'sns_sns' in context:
            self._list_function = "list_subscriptions_by_topic"
            parameters["TopicArn"] = context["sns_sns"]["TopicArn"]
        return parameters
